#ifndef IMAGE_RGB_INC
#define IMAGE_RGB_INC

#include "Image.h"

namespace Torch {

/** This class is designed to handle rgb images

    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 1.0
*/
class ImageRgb : public Image
{
public:
   	/**@name constructors and destructor */
        //@{
	/// Creates an empty rgb image 
    	ImageRgb();

	/** Creates a rgb image from a file.
	
	    @param filename_ is the filename of the image to load
	*/
    	ImageRgb(const char *filename_);

	/** Creates a black rgb image.
	
	    @param width_ is the width of the image
	    @param height_ is the height of the image
	*/
    	ImageRgb(int width_, int height_);

	///
	virtual ~ImageRgb();
	//@}
	
   	/**@name copy methods */
        //@{
	/** Copy from an image (in bytes).
	
	    @param width_ is the width of the image to copy
	    @param height_ is the height of the image to copy
	    @param pixmap_ is the image (in bytes) to copy into pixmap and data
	    @param coding_ is the coding of the #pixmap_# to copy
	*/
	virtual bool copyFrom(int width_, int height_, unsigned char *pixmap_, const char *coding_);

	/** Copy from an image (in real).
	
	    @param width_ is the width of the image to copy
	    @param height_ is the height of the image to copy
	    @param data_ is the image (in real) to copy into pixmap and data
	    @param coding_ is the coding of the #data_# to copy
	    @param max_ is the range of pixel values in #data_#
	*/
	virtual bool copyFrom(int width_, int height_, real *data_, const char *coding_, real max_ = 255.0);

	/** Copy an image from another image.
	
	    @param image_ is the image to copy
	*/
	virtual bool copyFrom(Image *image_);
	//@}
	
   	/**@name load/save methods */
        //@{
	/// save the image
	virtual void saveXFile(XFile *file);

	/// load the image
	virtual void loadXFile(XFile *file);
	//@}

   	/**@name drawing methods */
        //@{
	/** draw a pixel in the image.
	
	    @param x_ is the #x# coordinate of the pixel to draw
	    @param y_ is the #y# coordinate of the pixel to draw
	    @param c is the color to draw
	*/
	virtual void drawpixel(int x_, int y_, Color c);
	//@}
};

}

#endif
