#ifndef GIF_DISK_X_FILE_INC
#define GIF_DISK_X_FILE_INC

#include "ImageDiskXFile.h"

namespace Torch {

#define MAXCOLORMAPSIZE         256
  
struct gifscreen
{
        unsigned int    Width;
        unsigned int    Height;
        unsigned char   ColorMap[3][MAXCOLORMAPSIZE];
        unsigned int    BitPixel;
        unsigned int    ColorResolution;
        unsigned int    Background;
        unsigned int    AspectRatio;
};

struct gif89
{
        int transparent;
        int delayTime;
        int inputFlag;
        int disposal;
};

/** This class is designed to handle a GIF image file on the disk
    
    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 1.0
*/
class gifDiskXFile : public ImageDiskXFile
{
	gifscreen  GifScreen;
	gif89      Gif89;

	int imageNumber;
	int ZeroDataBlock;

	int ReadColorMap(int number, unsigned char buffer[3][MAXCOLORMAPSIZE]);
	int DoExtension(int label);
	int GetDataBlock(unsigned char  *buf);
	int GetCode(int code_size, int flag);
	int LWZReadByte(int flag, int input_code_size);

	bool read_rgbimage_from_gif(int width, int height, unsigned char cmap[3][MAXCOLORMAPSIZE], int interlace);

public:
   	/**@name constructor and destructor */
        //@{
	/** Makes a ImageDiskXFile from an filename.
	
	    @param file_name is the filename of the image
	    @param open_flags is the opening mode of the file
	*/
	gifDiskXFile(const char *file_name, const char *open_flags);

	/// Destructor
    	virtual ~gifDiskXFile();
	//@}
	
   	/**@name read/write image header and image pixmap */
        //@{
	/** read the header from the file.
	
	    @param width_ is the width of the image read in the header
	    @param height_ is the height of the image read in the header
	*/
	virtual bool readHeader(int *width_, int *height_);

	/// read the #pixmap# from the file
	virtual int readPixmap();

	/** write the header from the file.
	
	    @param width_ is the width of the image written in the header
	    @param height_ is the height of the image written in the header
	*/
	virtual bool writeHeader(int width_, int height_);

	/// write the #pixmap_# into the file
	virtual int writePixmap(unsigned char *pixmap_);
	//@}
};

}

#endif
