#include "ipBlock.h"

namespace Torch {

void ipBlock::init_(void)
{	
	pixeladv = block_size - block_overlap;
	
	rowblocks = 0;
	colblocks = 0;
	int row_, col_;

	for(row_ = 1 ; row_ <= height-(block_size-1) ; row_ += pixeladv) 
		rowblocks++;
	
	for(col_ = 1 ; col_ <= width-(block_size-1) ; col_ += pixeladv) 
		colblocks++;
		
	int end_row = row_ - pixeladv + (block_size-1);
	int end_col = col_ - pixeladv + (block_size-1);

	int rem_rows = height - end_row;
	int rem_cols = width - end_col;
	
	row_offset = rem_rows/2;
	col_offset = rem_cols/2;

	n_block = rowblocks * colblocks;

	subimage = new(allocator) Sequence(1, n_planes*block_size*block_size);

	if(verbose)
        {
                print("ipBlock::init()\n");
                print("   row_offset = %d\n", row_offset);
                print("   col_offset = %d\n", col_offset);
                print("   rows = %d\n", height);
                print("   cols = %d\n", width);
                print("   block size = %d\n", block_size);
                print("   block overlap = %d\n", block_overlap);
                print("");
                print("   n_block   = %d\n", n_block);
                print("   rowblocks = %d\n", rowblocks);
                print("   colblocks = %d\n", colblocks);
        }
}

ipBlock::ipBlock(int width_, int height_, const char *coding_, ipCore *ip_, int ip_output_size_, int block_size_, int block_overlap_) : ipCore(coding_) 
{
	width = width_;
	height = height_;
	
	ip = ip_;

	
	block_size = block_size_;
	block_overlap = block_overlap_;

	if(block_size < 1)
		error("ipBlock: block size (%d) incorrect.", block_size);   	
	
	if((block_overlap < 0) || (block_overlap > (block_size-1)))
		error("ipBlock: overlap should be in range [0...%d].", block_size-1);   	
	// set intern tmp variables (and n_block)
	init_();

	seq_out = new(allocator) Sequence(n_block, ip_output_size_);
}

void ipBlock::process(Sequence *seq_in)
{
	for(int rowblock=0; rowblock<rowblocks; rowblock++)
	{
	   	for(int colblock=0; colblock<colblocks; colblock++) 
		{
			int row = row_offset + rowblock*pixeladv;
			int col = col_offset + colblock*pixeladv;

			// Extract a block of the input image
			switch(coding_id)
			{
			case 0:
				cropGray(seq_in->frames[0], width, height, col, row, block_size, block_size, subimage->frames[0]);
				break;
			case 1:
				cropRgb(seq_in->frames[0], width, height, col, row, block_size, block_size, subimage->frames[0]);
				break;
			case 2:
				message("ipBlock::process() not implemented for YUV format");
				break;
			case 3:
				message("ipBlock::process() not implemented for float format");
				break;
			}

			// process the block 
			ip->process(subimage);

			// Storing blocks as [colblock][rowblock] (i.e. [x][y])
		   	int index_block = colblock + rowblock * colblocks;

			// Copy the output of the pattern machine
			for(int i=0 ; i < seq_out->frame_size ; i++) 
			    seq_out->frames[index_block][i] = ip->seq_out->frames[0][i];
		}
	}
}

ipBlock::~ipBlock()
{
}

}

