#include "ipDCT2Dinverse.h"

namespace Torch {

ipDCT2Dinverse::ipDCT2Dinverse(int dct_dim_, int block_size_) : ipCore("gray")
{
	dct_dim = dct_dim_;
	block_size = block_size_;

	if(block_size >  DCT2D_MAX_BLOCK)
	   	error("iDCT2D(): The block size is too big.");

	dct = new(allocator) ipDCT2D(block_size, "gray", dct_dim);

	seq_out = new(allocator) Sequence(1, block_size*block_size);

	if(coding_id != 0)
                warning("inverse 2D DCT not implemented for rgb, yuv and float coding yet.");
}

void ipDCT2Dinverse::process(Sequence *seq_in)
{
	for(int y = 0 ; y < block_size ; y++)  
	{
	   	for(int x = 0 ; x < block_size ; x++)  
	   	{
		   	int id_ = x + y * block_size;
			
			double sum = 0.0;
		
			for(int i = 0 ; i < dct_dim ; i++) 
			{
			   	int u = dct->dct_coeff_order[2*i];
				int v = dct->dct_coeff_order[2*i+1];
			   	
				sum += dct->alpha[u][v] * seq_in->frames[0][i] * dct->cosine[u][v][x][y];

			}

			seq_out->frames[0][id_] = sum;
		}
	}
}

ipDCT2Dinverse::~ipDCT2Dinverse()
{
}

}

