#include "ipIntegralImage.h"

namespace Torch {

ipIntegralImage::ipIntegralImage(int width_, int height_, const char *coding_, bool square_) : ipCore(coding_)
{
	width = width_;
        height = height_;

	square = square_;

	if(square) seq_out = new(allocator) Sequence(2, width*height*n_planes);
	else seq_out = new(allocator) Sequence(1, width*height*n_planes);
}

void ipIntegralImage::process(Sequence *seq_in)
{
	switch(coding_id)
	{
	case 0:
		if(square)
		{
		    real *src = seq_in->frames[0];
		    real *dst = seq_out->frames[0];
		    real *dst_sq = seq_out->frames[1];

		    dst[0] = src[0];
		    dst_sq[0] = src[0] * src[0];
			
                    // first line
                    for(int x = 1; x < width; x++)
		    {
                        dst[x] = dst[x-1] + src[x];
			
			real z = src[x];
			dst_sq[x] = dst_sq[x-1] + z*z;
		    }

                    for(int y = 1; y < height; y++)
                    {
                        real line = src[y*width];
			
			real z = src[y*width];
			real line_sq = z*z;

                        // first element of line y
                        dst[y*width] = dst[(y-1)*width] + line;

			dst_sq[y*width] = dst_sq[(y-1)*width] + line_sq;

                        for(int x = 1; x < width; x++)
                        {
                                line += src[y*width + x];
                                dst[y*width + x] = dst[(y-1)*width + x] + line;

				real z = src[y*width + x];
				line_sq += z*z;
				dst_sq[y*width + x] = dst_sq[(y-1)*width + x] + line_sq;
                        }
		    }
                }
		else
		{ 
  		    real *src = seq_in->frames[0];
		    real *dst = seq_out->frames[0];

		    dst[0] = src[0];
			
                    // first line
                    for(int x = 1; x < width; x++)
                        dst[x] = dst[x-1] + src[x];

                    for(int y = 1; y < height; y++)
                    {
                        real line = src[y*width];

                        // first element of line y
                        dst[y*width] = dst[(y-1)*width] + line;

                        for(int x = 1; x < width; x++)
                        {
                                line += src[y*width + x];
                                dst[y*width + x] = dst[(y-1)*width + x] + line;
                        }
		    }
		}

		break;
		
	case 1:
		message("ipIntegralImage::process() not implemented for RGB format");
		break;
		
	case 2:
		message("ipIntegralImage::process() not implemented for YUV format");
		break;
		
	case 3:
		message("ipIntegralImage::process() not implemented for float format");
		break;
	}
}

ipIntegralImage::~ipIntegralImage()
{
}

real iisum_check_limit(real *ii, int width, int height, int x, int y, int w, int h)
{
   	real P1, P2, P3, P4;
	
	if(x == 0 || y == 0) P1 = 0;
	else P1 = ii[x-1 + (y-1) * width];
	
	if(y == 0) P2 = 0;
	else P2 = ii[x + (w-1) + (y-1) * width];

	if(x == 0) P3 = 0;
	else P3 = ii[x-1 + (y + h-1) * width];

        P4 = ii[(x + w-1) + (y + h-1) * width];

	return P4 - P3 - P2 + P1;	
}

// Warning: to use only for x>=1 and y>=1
real iisum(real *ii, int width, int height, int x, int y, int w, int h)
{
   	real P1, P2, P3, P4;
	
	P1 = ii[x-1 + (y-1) * width];
	P2 = ii[x + (w-1) + (y-1) * width];
	P3 = ii[x-1 + (y + h-1) * width];
        P4 = ii[(x + w-1) + (y + h-1) * width];

	return P4 - P3 - P2 + P1;	
}

}

