#include "ipNormMeanLight.h"

namespace Torch {

ipNormMeanLight::ipNormMeanLight(int width_, int height_, const char *coding_) : ipCore(coding_)
{
	width = width_;
        height = height_;

	seq_out = new(allocator)Sequence(2, width_*height_*n_planes);
	
	for(int i=0; i<seq_out->frame_size; i++)
	{
		seq_out->frames[0][i] = 0.0;
		seq_out->frames[1][i] = 0.0;
	}
}

void ipNormMeanLight::process(Sequence *seq_in)
{
	int input_size = width * height;
	
	real *src = seq_in->frames[0];
	real *dst = seq_out->frames[0];
	real *linearplane = seq_out->frames[1];
	real mean_src;
	
	switch(coding_id)
	{
	case 0: // gray
	   	mean_src = 0.0;
		for(int h = 0; h < height; h++)
		{
			for(int w = 0; w < width; w++)
			{
				int i = h * width + w;
				mean_src += src[i];
			}
		}
		mean_src /= input_size;
	
		// Normalize 
		if(verbose) 
			print(" + mean src = %g\n", mean_src);
	
		for(int h = 0; h < height; h++)
		{
			for(int w = 0; w < width; w++)
			{
				int i = h * width + w;
				linearplane[i] = mean_src;
				dst[i] = 128.0 * src[i] / mean_src;
			}
		}
		break;
	case 1: 
	case 2: 
		message("ipNormMeanLight::process() not implemented for RGB and YUV format");
		break;
	case 3: 
	   	mean_src = 0.0;
		for(int h = 0; h < height; h++)
		{
			for(int w = 0; w < width; w++)
			{
				int i = h * width + w;
				mean_src += src[i];
			}
		}
		mean_src /= input_size;
	
		// Normalize 
		if(verbose) 
			print(" + mean src = %g\n", mean_src);
	
		for(int h = 0; h < height; h++)
		{
			for(int w = 0; w < width; w++)
			{
				int i = h * width + w;
				linearplane[i] = mean_src;
				dst[i] = src[i] / mean_src;
			}
		}
		break;
	}
}

ipNormMeanLight::~ipNormMeanLight()
{
}

}

