#ifndef IPZOOM_INC
#define IPZOOM_INC

#include "ipCore.h"

namespace Torch {

/** This class is designed to zoom an image

    \begin{verbatim}

                         +--------------+         +----+
           +--+          |              |         |XXXX|
	   |XX|   ---->  |    ipZoom    | ---->   |XXXX|
           |XX|          |              |         |XXXX|
           +--+          +--------------+         +----+

	 image                                  zoomed image 
	 
    \end{verbatim}
    
    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 2.0
*/
class ipZoom : public ipCore
{
   	// width of the input image
	int width;

	// height of the input image
	int height;

	// width of the output image
	int width_out;

	// height of the output image
	int height_out;

	// zoom factor
   	int zoom_factor;

public:
	/** constructor
	
	    @param width_ is the #width# of the input image
	    @param height_ is the #height# of the input image
	    @param coding_ is the #coding# format of the input image
	    @param zoom_factor_ is the zoom factor
	*/
    	ipZoom(int width_, int height_, const char *coding_, int zoom_factor_);

	///
	virtual ~ipZoom();

	///
	virtual void process(Sequence *seq_in);

	//-----
	
	///
	virtual int getWidthOut(void);

	///
        virtual int getHeightOut(void);
};

}

#endif
