const char *help = "\
progname: smoothpgm.cc\n\
code2html: This program reads a pgm image and smooth it.\n\
version: Torch3 vision2.0, 2004-2005\n\
(c) Sebastien Marcel (marcel@idiap.ch)\n";

#include "ImageGray.h"
#include "ipSmoothGaussian3.h"
#include "DiskXFile.h"
#include "CmdLine.h"

using namespace Torch;

int main(int argc, char **argv)
{
	char *image_filename;
	bool verbose;
	real sigma;
  
	// Construct the command line
  	CmdLine cmd;
	cmd.setBOption("write log", false);
	
  	// Put the help line at the beginning
  	cmd.info(help);

  	cmd.addText("\nArguments:");
  	cmd.addSCmdArg("image filename", &image_filename, "image filename");
  	cmd.addText("\nOptions:");
  	cmd.addBCmdOption("-verbose", &verbose, false, "verbose");
  	cmd.addRCmdOption("-sigma", &sigma, 0.25, "sigma");

	cmd.read(argc, argv);


  	DiskXFile *image_file = NULL;
	Image *image_in = NULL;
	Image *image_out = NULL;

	image_in = new ImageGray();
	image_in->setBOption("verbose", verbose);
	
	image_file = new DiskXFile(image_filename, "r");
	image_in->loadXFile(image_file);
	delete image_file;

	if(verbose)
	{
		print("Image info:\n");
		print("   width = %d\n", image_in->width);
		print("   height = %d\n", image_in->height);
		print("   format = %s (%d)\n", image_in->coding, image_in->n_planes);
	}

	ipCore *imachine = NULL;

	imachine = new ipSmoothGaussian3(image_in->width, image_in->height, "gray", sigma);
	imachine->setBOption("verbose", verbose);
	imachine->process(image_in);
	
	image_out = new ImageGray();
	image_out->setBOption("verbose", verbose);

	image_out->copyFrom(image_in->width, image_in->height, imachine->seq_out->frames[0], "gray");
	
	image_file = new DiskXFile("smooth.pgm", "w");
	image_out->saveXFile(image_file);
	delete image_file;

	delete imachine;
	delete image_out;
	delete image_in;

	return(0);
}
