const char *help = "\
progname: subsamplepgm.cc\n\
code2html: This program reads a pgm image and subsample it.\n\
version: Torch3 vision2.0, 2004-2005\n\
(c) Sebastien Marcel (marcel@idiap.ch)\n";

#include "ImageGray.h"
#include "ipSubsample.h"
#include "ipScale_yx.h"
#include "DiskXFile.h"
#include "CmdLine.h"

using namespace Torch;

int main(int argc, char **argv)
{
	char *image_filename;
	int width_out;
	int height_out;
	bool verbose;
	bool yx;
  
  	
	CmdLine cmd;
	cmd.setBOption("write log", false);
  	cmd.info(help);
  	cmd.addText("\nArguments:");
  	cmd.addSCmdArg("image filename", &image_filename, "image filename");
  	cmd.addICmdArg("new width", &width_out, "new width");
  	cmd.addICmdArg("new height", &height_out, "new height");
  	cmd.addText("\nOptions:");
  	cmd.addBCmdOption("-verbose", &verbose, false, "verbose");
  	cmd.addBCmdOption("-yx", &yx, false, "use yx subsampling");
	cmd.read(argc, argv);


  	DiskXFile *image_file = NULL;
	Image *image_in = NULL;
	Image *image_out = NULL;

	image_in = new ImageGray();
	image_in->setBOption("verbose", verbose);
	
	image_file = new DiskXFile(image_filename, "r");
	image_in->loadXFile(image_file);
	delete image_file;

	if(verbose)
	{
		print("Image info:\n");
		print("   width = %d\n", image_in->width);
		print("   height = %d\n", image_in->height);
		print("   format = %s (%d)\n", image_in->coding, image_in->n_planes);
	}

	ipCore *submachine = NULL;

	if(yx) submachine = new ipScale_yx(image_in->width, image_in->height, "gray", width_out, height_out);
	else submachine = new ipSubsample(image_in->width, image_in->height, "gray", width_out, height_out);
	submachine->setBOption("verbose", verbose);
	submachine->process(image_in);
	
	image_out = new ImageGray();
	image_out->setBOption("verbose", verbose);

	image_out->copyFrom(width_out, height_out, submachine->seq_out->frames[0], "gray");
	
	image_file = new DiskXFile("subsample.pgm", "w");
	image_out->saveXFile(image_file);
	delete image_file;

	delete submachine;
	delete image_out;
	delete image_in;

	return(0);
}
