#include "MlpCascadePatternDetect.h"
#include "PyramidWindowScan.h"
#include "ipSWmlpCascade.h"

namespace Torch {

MlpCascadePatternDetect::MlpCascadePatternDetect(	
		const char *model_filename_,
      		real threshold_,
      		int width_image_, int height_image_, 
		int patternWmin_, int patternHmin_, 
		int Wmin_, int Wmax_, real stepx_, real stepy_,
		real scale_factor_,
		real minMean_, real maxMean_, real minStdv_, real maxStdv_,
		real surfoverlap_fusion_,
		real threshold_activation_,
		real *pruning_ii_,
		real pruning_min_threshold_,
		real pruning_max_threshold_,
		int max_pattern_,
		bool verbose_)
   	: PatternDetect(width_image_, height_image_, 
			patternWmin_, patternHmin_, 
		      	Wmin_, Wmax_, stepx_, stepy_, scale_factor_, 
			minMean_, maxMean_, minStdv_, maxStdv_,
			pruning_ii_, pruning_min_threshold_, pruning_max_threshold_,
			max_pattern_, verbose_)
{
	ip_sw = new(allocator) ipSWmlpCascade(	width_image_, height_image_, 
						patternWmin_, patternHmin_,
						model_filename_, threshold_, verbose);
   
        int width_pattern = Wmin;
	
        for(int i = 0 ; i < n_levels ; i++)
        {
	   	if(width_pattern > Wmax) warning("MlpCascadePatternDetect::MlpCascadePatternDetect() width_pattern > Wmax");
		   
                int height_pattern = FixI((real) width_pattern * ratioPattern);
                if(verbose) print(" -> %dx%d\n", width_pattern, height_pattern);

                real scale = (real)width_pattern / patternWmin;

                int width_subsample_image = FixI((real)width_image / scale);
                int height_subsample_image = FixI((real)width_subsample_image * ratioImage);

                levels[i] = new(allocator) PyramidWindowScan(
                                width_subsample_image, height_subsample_image,
                                width_image, height_image,
                                patternWmin, patternHmin,
                                width_pattern, height_pattern,
                                stepx_factor, stepy_factor,
                                ip_sw,
                                patterncollector,
                                scale,
                                false);

                width_pattern = FixI((real)width_pattern * scale_factor);
        }

	merger = new(allocator) meanRectPatternMerger();
	patternfusion = new(allocator) NonOrientedPatternFusion(merger, 0, max_pattern_);
	surfoverlap_fusion = surfoverlap_fusion_;
	threshold_activation = threshold_activation_;
}
	
void MlpCascadePatternDetect::mergepatterns(void)
{
	if(patterncollector->n_patterns > 1)
	{
		qsort(patterncollector->patterns, patterncollector->n_patterns, sizeof(Pattern *), sortPatternConfDec);
		
		patternfusion->doOverlap(patterncollector->n_patterns, patterncollector->patterns, surfoverlap_fusion, ip_sw->threshold, true);

		if(verbose) print("Number of patterns after fusion (conf >= %g, overlap >= %g): %d\n", ip_sw->threshold, surfoverlap_fusion, patternfusion->n_patterns);

		qsort(patternfusion->patterns, patternfusion->n_patterns, sizeof(Pattern *), sortPatternConfDec);

		n_patterns = patternfusion->n_patterns;
		patterns = (Pattern **) allocator->alloc(n_patterns * sizeof(Pattern*));

		int P = n_patterns;
		n_patterns = 0;
		
		for(int i = 0 ; i < P ; i++)
		{
		   	if(patternfusion->patterns[i]->activation >= threshold_activation)
				patterns[n_patterns++] = patternfusion->patterns[i];
		}
	}
	else
	{
		n_patterns = patterncollector->n_patterns;
		patterns = patterncollector->patterns;
	}
}

MlpCascadePatternDetect::~MlpCascadePatternDetect()
{
}

}
