#ifndef MY_MLP_INC
#define MY_MLP_INC

#include "ConnectedMachine.h"
#include "MeanVarNorm.h"

namespace Torch {

/** This class is designed to handle a customized Multi-Layer Perceptron. 

    This is a clone of the MLP of Ronan Collobert but with an auto-load function.

    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 1.0
*/
class MyMLP : public ConnectedMachine
{
	//
	int *layerformat;
	int *nhu;

	//
	real *params_init;
	
	//
	void dump_params_init();

public:
	/// verbose
   	bool verbose;

	/// layers of the MLP
	GradientMachine **layers;

	/// number of layer
    	int n_layers;

	/// index of linear layers	
    	bool *is_linear;

	/** @name Lots of constructors */
	//@{
    	/** create a MLP with #n_layers# layers and #n_inputs_# inputs
	 
            The definitions of the layer come then: it's a string
	    followed by an integer for the number of outputs of the layer.
            valid strings are "linear", "tanh", "sigmoid", "softmax", "log-softmax",
	    "exp" and "softplus".

		\begin{verbatim}
        	Example: 
		         // create an MLP with one linear layer and one softmax layer
	        
		         MLP(2, n_inputs, "linear", n_outputs, "softmax", n_outputs);
		\end{verbatim}
    	*/
    	MyMLP(int n_layers, int n_inputs_, ...);

	/// create a perceptron (MLP without hidden layers)
    	MyMLP(int n_inputs_, char *layer_, int n_outputs_);

	/// create a MLP from a file
	MyMLP(const char *filename_);

	/// create an empty MLP
	MyMLP();
	//@}
	
    	// set the weight decay in all linear layers
    	void setWeightDecay(real weight_decay);

	/// reset the weights of the MLP
	virtual void reset();
	
	///
	virtual void loadXFile(XFile *file);

	///
	virtual void saveXFile(XFile *file);

	///
	void load(const char *filename_, PreProcessing *preprocess_norm=NULL);

	///
	void save(const char *filename_, PreProcessing *preprocess_norm=NULL);

	///
	void info();
    
	///
    	virtual ~MyMLP();
};

}

#endif
