#include "PCAMachine.h"

#include "DiskXFile.h"

namespace Torch {

PCAMachine::PCAMachine(int n_inputs_) : EigenMachine(n_inputs_)
{
   	Xm = NULL;
  	Xm = (real *) allocator->alloc(sizeof(real) * n_inputs);

    	input_ = NULL;
  	input_ = (real *) allocator->alloc(sizeof(real) * n_inputs);
}

void PCAMachine::forward(Sequence *inputs)
{
   	if(verbose_level >= 3) print("PCAMachine::forward() %d->%d ...\n", n_inputs, n_outputs);
	
    	for(int i = 0 ; i < n_inputs ; i++)
       		input_[i] = inputs->frames[0][i] - Xm[i];

	for(int i = 0 ; i < n_outputs ; i++)
	{
	   	outputs->frames[0][i] = 0.0;
		
       		for(int j = 0 ; j < n_inputs ; j++)
           		outputs->frames[0][i] += eigenvectors[j][i] * input_[j];
	}
}

void PCAMachine::loadXFile(XFile *file)
{
   	if(verbose_level >= 2) print("PCAMachine::loadXFile() ...\n");

   	EigenMachine::loadXFile(file);
	file->taggedRead(Xm, sizeof(real), n_inputs, "MEANS");
}

void PCAMachine::saveXFile(XFile *file)
{
   	if(verbose_level >= 2) print("PCAMachine::loadXFile() ...\n");

   	EigenMachine::saveXFile(file);
	file->taggedWrite(Xm, sizeof(real), n_inputs, "MEANS");
}

PCAMachine::~PCAMachine()
{
}

}
