#include "TangentDistance.h"
#include "vision.h"

namespace Torch {

//
mTangentDistance::mTangentDistance(int width_, int height_) : Metric(width_ * height_)
{
   	width = width_;
	height = height_;
	
	additiveBrightnessValue = 0.1;

	maxNumTangents = 9;

	choice = (int *) allocator->alloc(maxNumTangents*sizeof(int));
	choice[0] = 1;
	choice[1] = 1;
	choice[2] = 1;
	choice[3] = 1;
	choice[4] = 1;
	choice[5] = 1;
	choice[6] = 0;
	choice[7] = 0;
	choice[8] = 0;

	background = 0.0;
}

real mTangentDistance::measure(real *sample_, real *template_)
{
   	real d;
	
   	d = tangentDistance(sample_, template_);
	
	return d;
}

real mTangentDistance::tangentDistance(real* imageOne, real* imageTwo)
{
	int i, numTangents=0, numTangentsRemaining;
	double **tangents, dist;

	int size=width*height;

	for(i=0;i<maxNumTangents;++i)
 		if(choice[i]>0) numTangents++;

	tangents = (double **)allocator->alloc(numTangents*sizeof(double *));
	
	for(i=0;i<numTangents;++i) 
    		tangents[i] = (double *)allocator->alloc(size*sizeof(double));
  
	// determine the tangents of the first image
	calculateTangents(imageOne, tangents, numTangents);

	// find the orthonormal tangent subspace 
	numTangentsRemaining = normalizeTangents(tangents, numTangents);

	// determine the distance to the closest point in the subspace
	dist = calculateDistance(imageOne, imageTwo, tangents, numTangentsRemaining);

	for(i=0;i<numTangents;++i) 
    		allocator->free(tangents[i]);
  	allocator->free(tangents);

	return dist;
}

int mTangentDistance::calculateTangents(real * image, double ** tangents, int numTangents)
{
	int j, k, ind, tangentIndex, maxdim;
	double tp, factorW, offsetW, factorH, factor, offsetH, halfbg;
	double *tmp, *x1, *x2, *currentTangent;
  
	int size=height*width;
	maxdim=(height>width)?height:width;

	tmp=(double *)allocator->alloc(maxdim*sizeof(double));
	x1=(double *)allocator->alloc(size*sizeof(double));
	x2=(double *)allocator->alloc(size*sizeof(double));
      
	factorW=((double)width*0.5);
	offsetW=0.5-factorW;
	factorW=1.0/factorW;

	factorH=((double)height*0.5);
	offsetH=0.5-factorH;
	factorH=1.0/factorH;

	factor=(factorH<factorW)?factorH:factorW; //min

	halfbg=0.5*background;

	/* x1 shift along width */
	/* first use mask 1 0 -1 */
	for(k=0; k<height; k++) 
	{
		/* first column */
		ind=iIndex(k,0,width);
		x1[ind]= halfbg - image[ind+1]*0.5;
		
		/* other columns */
		for(j=1; j<width-1;j++) 
		{
			ind=iIndex(k,j,width);
			x1[ind]=(image[ind-1]-image[ind+1])*0.5;
		}
		/* last column */
		ind=iIndex(k,width-1,width);
		x1[ind]= image[ind-1]*0.5 - halfbg;
	}
	
	/* now compute 3x3 template */
	/* first line */
	for(j=0;j<width;j++) 
	{
		tmp[j]=x1[j];
		x1[j]=templatefactor2*x1[j]+templatefactor1*x1[j+width];
	}
	
	/* other lines */
	for(k=1;k<height-1;k++)
	for(j=0;j<width;j++) 
	{
		ind=iIndex(k,j,width);
		tp=x1[ind];
		x1[ind]=templatefactor1*tmp[j]+templatefactor2*x1[ind]+ templatefactor1*x1[ind+width];
		tmp[j]=tp;
	}
	
	/* last line */
	for(j=0;j<width;j++) 
	{
		ind=iIndex(height-1,j,width);
		x1[ind]=templatefactor1*tmp[j]+templatefactor2*x1[ind];
	}
	
	/* now add the remaining parts outside the 3x3 template */
	/* first two columns */
	for(j=0;j<2;j++)
		for(k=0;k<height;k++) 
		{
			ind=iIndex(k,j,width);
			x1[ind]+=templatefactor3*background;
		}
	
	/* other columns */
	for(j=2;j<width;j++)
		for(k=0;k<height;k++) 
		{
			ind=iIndex(k,j,width);
			x1[ind]+=templatefactor3*image[ind-2];
		} 
	
	for(j=0;j<width-2;j++)
		for(k=0;k<height;k++) 
		{
			ind=iIndex(k,j,width);
			x1[ind]-=templatefactor3*image[ind+2];
		}
	
	/* last two columns*/
	for(j=width-2;j<width;j++)
		for(k=0;k<height;k++) 
		{
			ind=iIndex(k,j,width);
			x1[ind]-=templatefactor3*background;
		}

	
	/*x2 shift along height */
	/* first use mask 1 0 -1 */
	for(j=0; j<width;j++) 
	{
		/* first line */
		x2[j]= halfbg - image[j+width]*0.5;
		
		/* other lines */
		for(k=1; k<height-1; k++) 
		{
			ind=iIndex(k,j,width);
			x2[ind]=(image[ind-width]-image[ind+width])*0.5;
		}
		
		/* last line */
		ind=iIndex(height-1,j,width);
		x2[ind]= image[ind-width]*0.5 - halfbg;
	}

	/* now compute 3x3 template */
	/* first column */
	for(j=0;j<height;j++) 
	{
		ind=iIndex(j,0,width);
		tmp[j]=x2[ind];
		x2[ind]=templatefactor2*x2[ind]+templatefactor1*x2[ind+1];
	}
	
	/* other columns */
	for(k=1;k<width-1;k++)
		for(j=0;j<height;j++) 
		{
			ind=iIndex(j,k,width);
			tp=x2[ind];
			x2[ind]=templatefactor1*tmp[j]+templatefactor2*x2[ind]+templatefactor1*x2[ind+1];
			tmp[j]=tp;
		}
	
	/* last column */
	for(j=0;j<height;j++) 
	{
		ind=iIndex(j,width-1,width);
		x2[ind]=templatefactor1*tmp[j]+templatefactor2*x2[ind];
	}

	/* now add the remaining parts outside the 3x3 template */
	for(j=0;j<2;j++)
		for(k=0;k<width;k++) 
		{
			ind=iIndex(j,k,width);
			x2[ind]+=templatefactor3*background;
		} 
	
	for(j=2;j<height;j++)
		for(k=0;k<width;k++) 
		{
			ind=iIndex(j,k,width);
			x2[ind]+=templatefactor3*image[ind-2*width];
		} 
	
	for(j=0;j<height-2;j++)
		for(k=0;k<width;k++) 
		{
			ind=iIndex(j,k,width);
			x2[ind]-=templatefactor3*image[ind+2*width];
		}
	
	for(j=height-2;j<height;j++)
		for(k=0;k<width;k++)
		{
			ind=iIndex(j,k,width);
			x2[ind]-=templatefactor3*background;
		}

	/* now go through the tangents */

	tangentIndex=0;

	if(choice[0]>0)
	{  
		// horizontal shift
		currentTangent=tangents[tangentIndex];
		for(ind=0;ind<size;ind++) currentTangent[ind]=x1[ind];
		tangentIndex++;
	}

	if(choice[1]>0)
	{
		// vertical shift
		currentTangent=tangents[tangentIndex];
		for(ind=0;ind<size;ind++) currentTangent[ind]=x2[ind];
		tangentIndex++;
	}

	if(choice[2]>0)
	{
		// hyperbolic  1
		//    Vapnik book says this is "diagonal deformation" (error), 
		//    this is the "axis deformation" 

		currentTangent=tangents[tangentIndex];
		ind=0;
		for(k=0;k<height;k++)
			for(j=0;j<width;j++)
			{
				currentTangent[ind] = ((j+offsetW)*x1[ind] - (k+offsetH)*x2[ind])*factor;
				ind++;
			}
		tangentIndex++;
	}

	if(choice[3]>0)
	{  
		// hyperbolic  2, (description = inverse of hyperbolic 1)
		
	   	currentTangent=tangents[tangentIndex];
		ind=0;
		for(k=0;k<height;k++)  
			for(j=0;j<width;j++)
			{
				currentTangent[ind] = ((k+offsetH)*x1[ind] + (j+offsetW)*x2[ind])*factor;
				ind++;
			}
		tangentIndex++;
	}

	if(choice[4]>0)
	{  
		// scaling

	   	currentTangent=tangents[tangentIndex];
		ind=0;
		for(k=0;k<height;k++)
			for(j=0;j<width;j++) 
			{
				currentTangent[ind] = ((j+offsetW)*x1[ind] + (k+offsetH)*x2[ind])*factor;
				ind++;
			}
		tangentIndex++;
	}

	if(choice[5]>0)
	{
		// rotation

	   	currentTangent=tangents[tangentIndex];
		ind=0;
		for(k=0;k<height;k++)
			for(j=0;j<width;j++) 
			{
				currentTangent[ind] = ((k+offsetH)*x1[ind] - (j+offsetW)*x2[ind])*factor;
				ind++;
			}
		tangentIndex++;
	}

	if(choice[6]>0)
	{
		// line thickness

		currentTangent=tangents[tangentIndex];
		ind=0;
		for(k=0;k<height;k++)
		for(j=0;j<width;j++) 
		{
			currentTangent[ind] = x1[ind]*x1[ind] + x2[ind]*x2[ind];
			ind++;
		} 
		tangentIndex++;
	}

	if(choice[7]>0)
	{
		// additive brightness

		currentTangent=tangents[tangentIndex];
		for(ind=0;ind<size;ind++)
			currentTangent[ind] = additiveBrightnessValue; 
		tangentIndex++;
	}

	if(choice[8]>0)
	{  
		// multiplicative brightness
		currentTangent=tangents[tangentIndex];
		for(ind=0;ind<size;ind++)
			currentTangent[ind] = image[ind];
		tangentIndex++;
	}

	allocator->free(tmp);
	allocator->free(x1);
	allocator->free(x2);
  
	if(tangentIndex != numTangents) warning("tangentIndex (%d) != numTangents (%d) !!", tangentIndex, numTangents);

	return tangentIndex;
}

int mTangentDistance::normalizeTangents(double ** tangents, int numTangents)
{
	int size = height*width;
  
	orthonormalizePzero (tangents, numTangents, size);

	// here we always return the original number of tangents dimensions
	// "lost" in the normalization are set to zero vectors this is not
	// what was intended originally, but it works and is kept for
	// backward compatibility

	return numTangents;
}

/** Calculates the distance between two images given a set of orthonormalized tangents. */
real mTangentDistance::calculateDistance(real* imageOne, real* imageTwo, double** tangents, int numTangents)
{
	double dist=0.0,tmp;
	const double *tangents_k;
	int k,l;

	int size=height*width;

	// first calculate squared Euclidean distance
	for(l=0;l<size;++l)
	{
		tmp=imageOne[l]-imageTwo[l];
		dist+=tmp*tmp;
	}
  
	// then subtract the part within the subspace 
	for(k=0;k<numTangents;++k)
	{
		tangents_k=tangents[k];
		tmp=0.0;
		
		for(l=0;l<size;++l) tmp+=(imageOne[l]-imageTwo[l])*tangents_k[l];
		dist-=tmp*tmp;
	}

	return dist;
}



const double ortho_singular_threshold = 1e-9;
// if vector norms are smaller than this value, it is assumed that no
// orthonormal basis exists; DBL_MIN is way to small for this; 
// surely there exist better algorithms than this one w.r.t
// numerical stability

int mTangentDistance::orthonormalizePzero(double **A, int num, int dim)
{
  int n,m,d,dim1;
  double projection,norm,tmp;
  double projection1,projection2,projection3,projection4;
  double *A_n, *A_m;

  dim1=dim-dim%4;

  for (n=0; n<num; ++n) {
    A_n=(double*)A[n];
    for (m=0; m<n; ++m) {
      A_m=(double*)A[m];
      projection=0.0;
      projection1=0.0;
      projection2=0.0;
      projection3=0.0;
      projection4=0.0;
      for (d=0; d<dim1; d+=4) {
	projection1+=A_n[d]*A_m[d]; 
	projection2+=A_n[d+1]*A_m[d+1]; 
	projection3+=A_n[d+2]*A_m[d+2]; 
	projection4+=A_n[d+3]*A_m[d+3]; }
      projection=projection1+projection2+projection3+projection4;
      for (; d<dim; ++d) {
	projection+=A_n[d]*A_m[d];}
      for (d=0; d<dim1; d+=4) {
	A_n[d]-=projection*A_m[d];
	A_n[d+1]-=projection*A_m[d+1];
	A_n[d+2]-=projection*A_m[d+2];
	A_n[d+3]-=projection*A_m[d+3];}
      for (; d<dim; ++d) {
	A_n[d]-=projection*A_m[d];}
    }
    // normalize
    norm=0.0;
    for (d=0; d<dim; ++d) {
      tmp=A_n[d];
      norm+=tmp*tmp;}
    if (norm<ortho_singular_threshold) {
      norm=0.0;}
    else {
      norm=1.0/sqrt(norm);}
    for (d=0; d<dim; ++d) {
      A_n[d]*=norm;}
  }

  return 0;
}

}
