#ifndef CAMERA_CONTROL_INC
#define CAMERA_CONTROL_INC

#include "Serial.h"

namespace Torch {

/** This class is designed to control a pan/tilt/zoom camera using a serial interface

    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 2.0
*/
class cameraControl : public Serial
{
public:
	///
   	cameraControl(char *device_, char port_);

	///
	virtual ~cameraControl();

	///
	virtual int getPanSpeedMin() = 0;

	///
	virtual int getPanSpeedMax() = 0;

	///
	virtual void setPanSpeed(int pan_speed_) = 0;

	///
	virtual int getTiltSpeedMin() = 0;

	///
	virtual int getTiltSpeedMax() = 0;

	///
	virtual void setTiltSpeed(int tilt_speed_) = 0;

	/// return the power status of the camera
	virtual int getPowerStatus() = 0;

	/// set power ON
	virtual bool setPowerOn() = 0;

	/// set power OFF
	virtual bool setPowerOff() = 0;

	/// BackLight compensation ON
	virtual bool setBackLightOn() = 0;
	
	/// BackLight compensation OFF
	virtual bool setBackLightOff() = 0;
	
	/** @name Pan/Tilt controls */
	//@{
	///
	virtual bool goHome() = 0;
	
	///
	virtual bool up() = 0;

	///
	virtual bool down() = 0;

	///
	virtual bool left() = 0;

	///
	virtual bool right() = 0;

	///
	virtual bool upLeft() = 0;

	///
	virtual bool upRight() = 0;

	///
	virtual bool downLeft() = 0;

	///
	virtual bool downRight() = 0;

	///
	virtual bool stopPanTilt() = 0;

	///
	virtual bool goRelative(int x, int y, int width, int height) = 0;

	///
	virtual bool goAbsolute(int x, int y, int width, int height) = 0;

	///
	virtual bool goRelative(int pan, int tilt) = 0;

	///
	virtual bool goAbsolute(int pan, int tilt) = 0;
	//@}
	
	/** @name Zoom controls */
	//@{
	///
	virtual bool zoomTeleStd() = 0;

	///
	virtual bool zoomWideStd() = 0;

	///
	virtual bool stopZoom() = 0;
	//@}
};

}

#endif
